@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001-2002, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2004, MetaSlash Inc.  All rights reserved.
 
 """
 Configuration information for checker.
@@ -15,19 +15,23 @@ import time
 
 _RC_FILE = ".pycheckrc"
 CHECKER_VAR = '__pychecker__'
-_VERSION = '0.8.10'
+_VERSION = '0.8.14'
 
 _DEFAULT_BLACK_LIST = [ "Tkinter", "wxPython", "gtk", "GTK", "GDK", ]
 _DEFAULT_VARIABLE_IGNORE_LIST = [ '__version__', '__warningregistry__', 
-                                  '__all__', '__credits__',
-                                  '__author__', '__email__', ]
+                                  '__all__', '__credits__', '__test__',
+                                  '__author__', '__email__', '__revision__', ]
 _DEFAULT_UNUSED_LIST = [ '_', 'empty', 'unused', 'dummy', ]
 
+# All these options are on even if -e/--errors is used
+_ERRORS = { 'noEffect': 1, }
+
 _OPTIONS = (
     ('Major Options', [
  ('e', 0, 'errors', None, 'turn off all warnings which are not likely errors'),
  ( '', 0, 'complexity', None, 'turn off all warnings which are related to complexity'),
  ('F', 1, 'config', None, 'specify .pycheckrc file to use'),
+ ('',  0, 'quixote', None, 'support Quixote\'s PTL modules'),
      ]),
     ('Error Control', [
  ('i', 0, 'import', 'importUsed', 'unused imports'),
@@ -51,19 +55,26 @@ _OPTIONS = (
  ('2', 0, 'constCond', 'constantConditions', 'a constant is used in a conditional statement'),
  ('1', 0, 'constant1', 'constant1', '1 is used in a conditional statement (if 1: or while 1:)'),
  ( '', 0, 'stringiter', 'stringIteration', 'check if iterating over a string'),
+ ( '', 0, 'stringfind', 'stringFind', 'check improper use of string.find()'),
  ('A', 0, 'callattr', 'callingAttribute', 'Calling data members as functions'),
  ('y', 0, 'classattr', 'classAttrExists', 'class attribute does not exist'),
  ('S', 1, 'self', 'methodArgName', 'First argument to methods'),
+ ('',  1, 'classmethodargs', 'classmethodArgNames', 'First argument to classmethods'),
  ('T', 0, 'argsused', 'argumentsUsed', 'unused method/function arguments'),
  ('z', 0, 'varargsused', 'varArgumentsUsed', 'unused method/function variable arguments'),
  ('G', 0, 'selfused', 'ignoreSelfUnused', 'ignore if self is unused in methods'),
  ('o', 0, 'override', 'checkOverridenMethods', 'check if overridden methods have the same signature'),
+ ('',  0, 'special', 'checkSpecialMethods', 'check if __special__ methods exist and have the correct signature'),
  ('U', 0, 'reuseattr', 'redefiningFunction', 'check if function/class/method names are reused'),
  ('Y', 0, 'positive', 'unaryPositive', 'check if using unary positive (+) which is usually meaningless'),
  ('j', 0, 'moddefvalue', 'modifyDefaultValue', 'check if modify (call method) on a parameter that has a default value'),
  ( '', 0, 'changetypes', 'inconsistentTypes', 'check if variables are set to different types'),
  ( '', 0, 'unpack', 'unpackNonSequence', 'check if unpacking a non-sequence'),
  ( '', 0, 'unpacklen', 'unpackLength', 'check if unpacking sequence with the wrong length'),
+ ( '', 0, 'badexcept', 'badExceptions', 'check if raising or catching bad exceptions'),
+ ('4', 0, 'noeffect', 'noEffect', 'check if statement appears to have no effect'),
+ ('',  0, 'modulo1', 'modulo1', 'check if using (expr % 1), it has no effect on integers and strings'),
+ ('',  0, 'isliteral', 'isLiteral', "check if using (expr is const-literal), doesn't always work on integers and strings"),
      ]),
     ('Possible Errors', [
  ('r', 0, 'returnvalues', 'checkReturnValues', 'check consistent return values'),
@@ -74,6 +85,7 @@ _OPTIONS = (
  ( '', 0, 'emptyslots', 'emptySlots', 'check if __slots__ is empty'),
  ('D', 0, 'intdivide', 'intDivide', 'check if using integer division'),
  ('w', 0, 'shadow', 'shadows', 'check if local variable shadows a global'),
+ ('s', 0, 'shadowbuiltin', 'shadowBuiltins', 'check if a variable shadows a builtin'),
      ]),
     ('Security', [
  ( '', 0, 'input', 'usesInput', 'check if input() is used'),
@@ -84,6 +96,7 @@ _OPTIONS = (
  ('b', 1, 'blacklist', 'blacklist', 'ignore warnings from the list of modules\n\t\t\t'),
  ('Z', 1, 'varlist', 'variablesToIgnore', 'ignore global variables not used if name is one of these values\n\t\t\t'),
  ('E', 1, 'unusednames', 'unusedNames', 'ignore unused locals/arguments if name is one of these values\n\t\t\t'),
+ ( '', 0, 'deprecated', 'deprecated', 'ignore use of deprecated modules/functions'),
      ]),
     ('Complexity', [
  ('L', 1, 'maxlines', 'maxLines', 'maximum lines in a function'),
@@ -190,6 +203,7 @@ class Config :
         self.quiet = 0
         self.onlyCheckInitForMembers = 0
         self.printParse = 0
+        self.quixote = 0
 
         self.noDocModule = 0
         self.noDocClass = 0
@@ -220,13 +234,21 @@ class Config :
         self.inconsistentTypes = 0
         self.unpackNonSequence = 1
         self.unpackLength = 1
+        self.badExceptions = 1
+        self.noEffect = 1
+        self.deprecated = 1
+        self.modulo1 = 1
+        self.isLiteral = 1
+        self.stringFind = 1
 
         self.unusedNames = _DEFAULT_UNUSED_LIST
         self.variablesToIgnore = _DEFAULT_VARIABLE_IGNORE_LIST
         self.blacklist = _DEFAULT_BLACK_LIST
         self.ignoreStandardLibrary = 0
         self.methodArgName = 'self'
+        self.classmethodArgNames = ['cls', 'klass']
         self.checkOverridenMethods = 1
+        self.checkSpecialMethods = 1
 
         self.argumentsUsed = 1
         self.varArgumentsUsed = 1
@@ -248,10 +270,12 @@ class Config :
         self.checkImplicitReturns = 1
         self.intDivide = 1
         self.shadows = 1
+        self.shadowBuiltins = 1
         self.unaryPositive = 1
         self.modifyDefaultValue = 1
         self.usesExec = 0
         self.usesInput = 1
+        self.constAttr = 1
 
     def loadFile(self, filename) :
         suppressions = {}
@@ -262,7 +286,8 @@ class Config :
             for key, value in dict.items() :
                 if self.__dict__.has_key(key) :
                     self.__dict__[key] = value
-                elif key not in ('suppressions', 'suppressionRegexs') :
+                elif key not in ('suppressions', 'suppressionRegexs') and \
+                     key[0] != '_':
                     print "Warning, option (%s) doesn't exist, ignoring" % key
 
             suppressions = _getSuppressions('suppressions', dict, filename)
@@ -306,6 +331,11 @@ class Config :
                 elif longArg == 'quiet' :
                     quiet = 1
                     continue
+                elif longArg == 'quixote' :
+                    import quixote
+                    quixote.enable_ptl()
+                    self.quixote = 1
+                    continue
                 elif longArg == 'config' :
                     otherConfigFiles.append(value)
                     continue
@@ -328,6 +358,14 @@ class Config :
                     newValue = int(newValue)
                 elif memberType == type([]) :
                     newValue = string.split(newValue, ',')
+                elif memberType == type('') and \
+                     newValue[0] in '\'"':
+                        try:
+                            newValue = eval(newValue)
+                        except:
+                            msg = 'Invalid option parameter: %s for %s\n' % \
+                                  (`newValue`, arg)
+                            sys.stderr.write(msg)
                 setattr(self, member, newValue)
             elif arg[0:2] == '--' :
                 setattr(self, member, arg[2:5] != 'no-')
@@ -345,7 +383,7 @@ def errors_only(complexity = 0) :
     "Return {} of Config with all warnings turned off"
     dict = Config().__dict__
     for k, v in dict.items() :
-        if type(v) == type(0) and v >= complexity :
+        if type(v) == type(0) and v >= complexity and not _ERRORS.has_key(k):
             dict[k] = 0
     return dict
 
