@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2004, MetaSlash Inc.  All rights reserved.
 
 """
 Python byte code operations.
@@ -36,6 +36,12 @@ def UNPACK_SEQUENCE(op) :
     "Deal w/Python 1.5.2 (UNPACK_[LIST|TUPLE]) or 2.0 (UNPACK_SEQUENCE)"
     return op in (92, 93,)
 
+def IS_CONDITIONAL_JUMP(op):
+    return op in (111, 112)
+
+def IS_NOT(op):
+    return op == 12
+
 HAVE_ARGUMENT = 90
 EXTENDED_ARG = 143
 
@@ -49,6 +55,9 @@ _HAS_JABS = (113, 119,)
 _CMP_OP =  ('<', '<=', '==', '!=', '>', '>=', 'in', 'not in', 'is',
             'is not', 'exception match', 'BAD')
 
+EXCEPT_COMPARISON = 10
+IS_COMPARISON = 8
+
 def getOperand(op, func_code, oparg) :
     if op in _HAS_NAME :
         return func_code.co_names[oparg]
