@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001-2002, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2004, MetaSlash Inc.  All rights reserved.
 
 """
 Check python source code files for possible errors and print warnings
@@ -17,6 +17,7 @@ import imp
 import os
 import glob
 import traceback
+import re
 
 # see __init__.py for meaning, this must match the version there
 LOCAL_MAIN_VERSION = 1
@@ -85,26 +86,58 @@ def getModules(arg_list) :
             arg = glob.glob(arg)
         new_arguments.append(arg)
 
-    PY_SUFFIX = '.py'
-    PY_SUFFIX_LEN = len(PY_SUFFIX)
-
+    PY_SUFFIXES = ['.py']
+    PY_SUFFIX_LENS = [3]
+    if _cfg.quixote:
+        PY_SUFFIXES.append('.ptl')
+        PY_SUFFIX_LENS.append(4)
+        
     modules = []
     for arg in _flattenList(new_arguments) :
         # is it a .py file?
-        if len(arg) > PY_SUFFIX_LEN and arg[-PY_SUFFIX_LEN:] == PY_SUFFIX:
-            arg_dir = os.path.dirname(arg)
-            if arg_dir and not os.path.exists(arg) :
-                print 'File or pathname element does not exist: "%s"' % arg
-                continue
+        for suf, suflen in zip(PY_SUFFIXES, PY_SUFFIX_LENS):
+            if len(arg) > suflen and arg[-suflen:] == suf:
+                arg_dir = os.path.dirname(arg)
+                if arg_dir and not os.path.exists(arg) :
+                    print 'File or pathname element does not exist: "%s"' % arg
+                    continue
 
-            module_name = os.path.basename(arg)[:-PY_SUFFIX_LEN]
-            if arg_dir not in sys.path :
-                sys.path.insert(0, arg_dir)
-            arg = module_name
+                module_name = os.path.basename(arg)[:-suflen]
+                if arg_dir not in sys.path :
+                    sys.path.insert(0, arg_dir)
+                arg = module_name
         modules.append(arg)
 
     return modules
 
+def _q_file(f):
+    # crude hack!!!
+    # imp.load_module requires a real file object, so we can't just
+    # fiddle def lines and yield them
+    import tempfile
+    fd, newfname = tempfile.mkstemp(suffix=".py", text=True)
+    newf = os.fdopen(fd, 'r+')
+    os.unlink(newfname)
+    for line in f:
+        mat = re.match(r'(\s*def\s+\w+\s*)\[(html|plain)\](.*)', line)
+        if mat is None:
+            newf.write(line)
+        else:
+            newf.write(mat.group(1)+mat.group(3)+'\n')
+    newf.seek(0)
+    return newf
+
+def _q_find_module(p, path):
+    if not _cfg.quixote:
+        return imp.find_module(p, path)
+    else:
+        for direc in path:
+            try:
+                return imp.find_module(p, [direc])
+            except ImportError:
+                f = os.path.join(direc, p+".ptl")
+                if os.path.exists(f):
+                    return _q_file(file(f)), f, ('.ptl', 'U', 1)
 
 def _findModule(name) :
     """Returns the result of an imp.find_module(), ie, (file, filename, smt)
@@ -114,7 +147,7 @@ def _findModule(name) :
     packages = string.split(name, '.')
     for p in packages :
         # smt = (suffix, mode, type)
-        file, filename, smt = imp.find_module(p, path)
+        file, filename, smt = _q_find_module(p, path)
         if smt[-1] == imp.PKG_DIRECTORY :
             try :
                 # package found - read path info from init file
@@ -128,7 +161,7 @@ def _findModule(name) :
             # we need to choose the real (replaced) version
             if m.__name__ != p :
                 try :
-                    file, filename, smt = imp.find_module(m.__name__, path)
+                    file, filename, smt = _q_find_module(m.__name__, path)
                     m = imp.load_module(p, file, filename, smt)
                 finally :
                     if file is not None :
@@ -184,13 +217,20 @@ class Class :
         self.name = name
         self.classObject = getattr(module, name)
 
-        # Argh -- some ExtensionClasses don't have a __module__ attribute.
         modname = getattr(self.classObject, '__module__', None)
+        if modname is None:
+            # hm, some ExtensionClasses don't have a __module__ attribute
+            # so try parsing the type output
+            typerepr = repr(type(self.classObject))
+            mo = re.match("^<type ['\"](.+)['\"]>$", typerepr)
+            if mo:
+                modname = ".".join(mo.group(1).split(".")[:-1])
+
         self.module = sys.modules.get(modname)
         if not self.module:
             self.module = module
-            sys.stderr.write("warning: couldn't find real module for class %r "
-                             "(module name: %r)\n"
+            sys.stderr.write("warning: couldn't find real module for class %s "
+                             "(module name: %s)\n"
                              % (self.classObject, modname))
         self.ignoreAttrs = 0
         self.methods = {}
@@ -238,26 +278,29 @@ class Class :
             func_name = className + func_name
         return func_name
 
-    def addMethod(self, method, className = None, methodName = None) :
+    def addMethod(self, method, methodName = None) :
         if type(method) == types.StringType :
             self.methods[method] = None
-            return
-        if not hasattr(method, "func_name") :
-            return
-
-        if not methodName :
-            methodName = self.__getMethodName(method.func_name, className)
-        self.methods[methodName] = function.Function(method)
+        else :
+            assert methodName is not None, "must supply methodName"
+            self.methods[methodName] = function.Function(method, 1)
                 
     def addMethods(self, classObject) :
         for classToken in _getClassTokens(classObject) :
-            token = getattr(classObject, classToken)
-            if type(token) == types.MethodType :
-                self.addMethod(token.im_func, classObject.__name__, token.__name__)
+            token = getattr(classObject, classToken, None)
+            if token is None:
+                continue
+
+            # Looks like a method.  Need to code it this way to
+            # accommodate ExtensionClass and Python 2.2.  Yecchh.
+            if (hasattr(token, "func_code") and
+                hasattr(token.func_code, "co_argcount")): 
+                self.addMethod(token, token.__name__)
+
             elif hasattr(token, '__get__') and \
                  not hasattr(token, '__set__') and \
                  type(token) is not types.ClassType :
-                self.addMethod(token.__name__)
+                self.addMethod(getattr(token, '__name__', classToken))
             else :
                 self.members[classToken] = type(token)
                 self.memberRefs[classToken] = None
@@ -270,11 +313,14 @@ class Class :
     def addMembers(self, classObject) :
         if not cfg().onlyCheckInitForMembers :
             for classToken in _getClassTokens(classObject) :
-                method = getattr(classObject, classToken)
+                method = getattr(classObject, classToken, None)
                 if type(method) == types.MethodType :
                     self.addMembersFromMethod(method.im_func)
-        elif hasattr(classObject, "__init__") :
-            self.addMembersFromMethod(classObject.__init__.im_func)
+        else:
+            try:
+                self.addMembersFromMethod(classObject.__init__.im_func)
+            except AttributeError:
+                pass
 
     def addMembersFromMethod(self, method) :
         if not hasattr(method, 'func_code') :
@@ -290,7 +336,7 @@ class Class :
                     stack.append(operand)
                 elif OP.STORE_ATTR(op) :
                     if len(stack) > 0 :
-                        if stack[-1] == cfg().methodArgName :
+                        if stack[-1] == cfg().methodArgName:
                             value = None
                             if len(stack) > 1 :
                                 value = type(stack[-2])
@@ -332,6 +378,17 @@ class Class :
                 result.append(m)
         return result
 
+def _getLineInFile(moduleName, linenum):
+    line = ''
+    file, filename, smt = _findModule(moduleName)
+    try:
+        lines = file.readlines()
+        line = string.rstrip(lines[linenum - 1])
+    except (IOError, IndexError):
+        pass
+    file.close()
+    return line
+
 def importError(moduleName):
     exc_type, exc_value, tb = sys.exc_info()
 
@@ -347,11 +404,25 @@ def importError(moduleName):
     # the clutter of a traceback most of the time.  Also, the locus of
     # the error is usually irrelevant for ImportError, so the lack of
     # traceback shouldn't be a problem.
-    if exc_type is not ImportError:
+    if exc_type is SyntaxError:
+        # SyntaxErrors are special, we want to control how we format
+        # the output and make it consistent for all versions of Python
+        e = exc_value
+        msg = '%s (%s, line %d)' % (e.msg, e.filename, e.lineno)
+        line = _getLineInFile(moduleName, e.lineno)
+        offset = e.offset
+        if type(offset) is not types.IntType:
+            offset = 0
+        exc_value = '%s\n    %s\n   %s^' % (msg, line, ' ' * offset)
+    elif exc_type is not ImportError:
         sys.stderr.write("  Caught exception importing module %s:\n" %
                          moduleName)
 
-        tbinfo = traceback.extract_tb(tb)
+        try:
+            tbinfo = traceback.extract_tb(tb)
+        except:
+            tbinfo = []
+            sys.stderr.write("      Unable to format traceback\n")
         for filename, line, func, text in tbinfo[1:]:
             sys.stderr.write("    File \"%s\", line %d" % (filename, line))
             if func != "?":
@@ -369,6 +440,14 @@ def importError(moduleName):
         sys.stderr.write('**error formatting exception value**\n')
 
 
+def _getPyFile(filename):
+    """Return the file and '.py' filename from a filename which could
+    end with .py, .pyc, or .pyo"""
+
+    if filename[-1] in 'oc' and filename[-4:-1] == '.py':
+        return filename[:-1]
+    return filename
+
 class Module :
     "Class to hold all information for a module"
 
@@ -404,8 +483,14 @@ class Module :
 
     def addClass(self, name) :
         self.classes[name] = c = Class(name, self.module)
-        packages = string.split(str(c.classObject), '.')
-        c.ignoreAttrs = packages[0] in cfg().blacklist
+        try:
+            objName = str(c.classObject)
+        except TypeError:
+            # this can happen if there is a goofy __getattr__
+            c.ignoreAttrs = 1
+        else:
+            packages = string.split(objName, '.')
+            c.ignoreAttrs = packages[0] in cfg().blacklist
         if not c.ignoreAttrs :
             self.__addAttributes(c, c.classObject)
 
@@ -423,26 +508,22 @@ class Module :
             self.modules[name] = module
 
     def filename(self) :
-        if not self.module :
-            return self.moduleName
         try :
             filename = self.module.__file__
         except AttributeError :
             filename = self.moduleName
-        if string.lower(filename[-4:]) == '.pyc' :
-            filename = filename[:-4] + '.py'
-        return filename
+        return _getPyFile(filename)
 
-    def load(self) :
+    def load(self):
         try :
             # there's no need to reload modules we already have
             module = sys.modules.get(self.moduleName)
             if module :
                 if not _allModules[self.moduleName].module :
-                    return self.initModule(module)
+                    return self._initModule(module)
                 return 1
 
-            return self.initModule(self.setupMainCode())
+            return self._initModule(self.setupMainCode())
         except (SystemExit, KeyboardInterrupt) :
             exc_type, exc_value, exc_tb = sys.exc_info()
             raise exc_type, exc_value
@@ -451,6 +532,19 @@ class Module :
             return 0
 
     def initModule(self, module) :
+        if not self.module:
+            filename = _getPyFile(module.__file__)
+            if string.lower(filename[-3:]) == '.py':
+                try:
+                    file = open(filename)
+                except IOError:
+                    pass
+                else:
+                    self._setupMainCode(file, filename, module)
+            return self._initModule(module)
+        return 1
+
+    def _initModule(self, module):
         self.module = module
         self.attributes = dir(self.module)
 
@@ -479,15 +573,17 @@ class Module :
     def setupMainCode(self) :
         file, filename, smt = _findModule(self.moduleName)
         # FIXME: if the smt[-1] == imp.PKG_DIRECTORY : load __all__
+        module = imp.load_module(self.moduleName, file, filename, smt)
+        self._setupMainCode(file, filename, module)
+        return module
+
+    def _setupMainCode(self, file, filename, module):
         try :
-            module = imp.load_module(self.moduleName, file, filename, smt)
             self.main_code = function.create_from_file(file, filename, module)
         finally :
             if file != None :
                 file.close()
 
-        return module
-
 
 def getAllModules() :
     "Returns a list of all modules that should be checked."
@@ -503,8 +599,10 @@ _BUILTIN_MODULE_ATTRS = { 'sys': [ 'ps1', 'ps2', 'tracebacklimit',
                                  ],
                         }
 
-def fixupBuiltinModules() :
+def fixupBuiltinModules(needs_init=0):
     for moduleName in sys.builtin_module_names :
+        if needs_init:
+            _ = Module(moduleName, 0)
         module = _allModules.get(moduleName, None)
         if module is not None :
             try :
@@ -516,7 +614,10 @@ def fixupBuiltinModules() :
                 module.attributes = [ '__dict__' ] + dir(m) + extra_attrs
 
 
-def _printWarnings(warnings, stream = sys.stdout) :
+def _printWarnings(warnings, stream=None):
+    if stream is None:
+        stream = sys.stdout
+    
     warnings.sort()
     lastWarning = None
     for warning in warnings :
@@ -575,6 +676,26 @@ def main(argv) :
         sys.stderr.write(_VERSION_MISMATCH_ERROR)
         sys.exit(100)
 
+    # remove empty arguments
+    argv = filter(None, argv)
+        
+    # if the first arg starts with an @, read options from the file
+    # after the @ (this is mostly for windows)
+    if len(argv) >= 2 and argv[1][0] == '@':
+        # read data from the file
+        command_file = argv[1][1:]
+        try:
+            f = open(command_file, 'r')
+            command_line = f.read()
+            f.close()
+        except IOError, err:
+            sys.stderr.write("Unable to read commands from file: %s\n  %s\n" % \
+                             (command_file, err))
+            sys.exit(101)
+
+        # convert to an argv list, keeping argv[0] and the files to process
+        argv = argv[:1] + string.split(command_line) + argv[2:]
+ 
     global _cfg
     _cfg, files, suppressions = Config.setupFromArgs(argv[1:])
     if not files :
@@ -610,15 +731,33 @@ if __name__ == '__main__' :
 else :
     _orig__import__ = None
     _suppressions = None
+    _warnings_cache = {}
+
+    def _get_unique_warnings(warnings):
+        for i in range(len(warnings)-1, -1, -1):
+            w = warnings[i].format()
+            if _warnings_cache.has_key(w):
+                del warnings[i]
+            else:
+                _warnings_cache[w] = 1
+        return warnings
+
+    def __import__(name, globals=None, locals=None, fromlist=None):
+        if globals is None:
+            globals = {}
+        if locals is None:
+            locals = {}
+        if fromlist is None:
+            fromlist = []
 
-    def __import__(name, globals={}, locals={}, fromlist=[]) :
         check = not sys.modules.has_key(name) and name[:10] != 'pychecker.'
         pymodule = _orig__import__(name, globals, locals, fromlist)
         if check :
             try :
                 module = Module(pymodule.__name__)
-                if module.load() :
-                    _printWarnings(warn.find([module], _cfg, _suppressions))
+                if module.initModule(pymodule):
+                    warnings = warn.find([module], _cfg, _suppressions)
+                    _printWarnings(_get_unique_warnings(warnings))
                 else :
                     print 'Unable to load module', pymodule.__name__
             except Exception:
@@ -632,12 +771,13 @@ else :
 
         args = string.split(os.environ.get('PYCHECKER', ''))
         _cfg, files, _suppressions = Config.setupFromArgs(args)
-        fixupBuiltinModules()
+        utils.initConfig(_cfg)
+        fixupBuiltinModules(1)
 
         # keep the orig __import__ around so we can call it
         import __builtin__
         _orig__import__ = __builtin__.__import__
         __builtin__.__import__ = __import__
 
-    if not os.environ.get('PYCHECKER_DISABLED', 0) :
+    if not os.environ.get('PYCHECKER_DISABLED') :
         _init()
