@@ -57,3 +57,15 @@ def uuu():
   a=1+2
   return a[0], [['a', 'b'], [1,2,3,4]]
 
+def renderElement(a, b=2, c=3, **kw):
+    contents = kw
+    print contents
+
+def renderItem(text):
+    return renderElement(1, contents=text, z=1, y=2, x=3)
+
+def test_kw_lambda(a, b, c):
+    return renderElement(a, ff=lambda value: '=' + renderItem(b))
+
+str = '53'
+_ = "blah, don't care, shouldn't warn"
