@@ -7,3 +7,6 @@ b = Bbb()
 def xxx():
     print func24(1)
 
+def func(shadow):
+    print shadow
+
