@@ -22,7 +22,7 @@ def func3(x):
     return 0
 
 def func4(x):
-    'should produce a warning'
+    'should not produce a warning'
     while 1:
         if x == 1:
             return 1
@@ -140,3 +140,20 @@ def func20(x):
             break
         return 1
 
+def func21(x):
+    'should not produce a warning'
+    try:
+        if x == 1:
+            return 3
+        return 6
+    finally:
+        print 'do nothing'
+
+def func22(x):
+    'should not produce a warning'
+    while 1:
+        for _ in range(10) :
+            x = x / 2
+            break
+        return 1
+
